@@ -13,8 +13,8 @@ shinyUI(fluidPage(
                 animate=animationOptions(200, loop=TRUE)),
       sliderInput("Slider2", min=1, max=60, step=1, value=1, label="Cumulative",
                   animate=animationOptions(200, loop=TRUE)),
-      rglcontrollerOutput('thecontroller'),
-      rglcontrollerOutput('thecontroller2'),
+      playwidgetOutput('thecontroller'),
+      playwidgetOutput('thecontroller2'),
       actionButton('newStart', 'Restart')),
     mainPanel(
       rglwidgetOutput('thewidget', width = "100%", height = 512))
